class Page

  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::Userstamp
  include Mongoid::Tree
  include Mongoid::Tree::Ordering

  store_in session: :bolton_cms

  embeds_many :fragments, cascade_callbacks: true
  embeds_many :assets,   cascade_callbacks: true

  accepts_nested_attributes_for :fragments, allow_destroy: true
  accepts_nested_attributes_for :assets,   allow_destroy: true

  field :slug,          type: String
  field :path,          type: String
  field :template,      type: Symbol
  field :title,         type: String
  field :keywords,      type: Array
  field :description,   type: String
  field :body,          type: String,  default: ''
  field :filter,        type: Symbol,  default: :markdown
  field :chronological, type: Boolean, default: false

  index({ path: 1 }, { unique: true })

  validates_associated  :parent, :children
  validates_presence_of :template
  validates_presence_of :title
  validate              :no_grandchildren, if: :chronological?
  validate              :no_children,      if: Proc.new { self.parent? && self.parent.chronological? }

  before_rearrange :set_slug
  after_rearrange  :rebuild_path
  before_save      :destroy_flagged_assets
  before_save      :flag_used_assets
  before_destroy   :move_children_to_parent

  def post?
    parent && parent.chronological?
  end

  def unused_assets
    assets.where(used: false)
  end

  def active_fragments
    fragments.where(active: true)
  end

  def inactive_fragments
    fragments.where(active: false)
  end

  def find_fragment(label)
    fragments.find_by(label: label) unless label.blank?
  end

  def manage_fragments(configured_fragments = [])
    fragments.each do |fragment|
      if fragment.active = configured_fragments.include?(fragment.label)
        configured_fragments.delete fragment.label
      end
    end

    configured_fragments.each do |fragment|
      fragments << Fragment.new(label: fragment)
    end

    inactive_fragments.where(body: '').destroy_all

    save
  end

  private

  def no_grandchildren
    child_parents = !!self.children.detect{ |child| child.children? }

    errors.add :children, 'Page with children may not be added to a chronological index' if child_parents
  end

  def no_children
    errors.add :children, 'Page may not have children if parent is a chronological index' unless self.children.empty?
  end

  def rebuild_path
    self.path = self.ancestors_and_self.collect(&:slug).join('/')

    self.rearrange_children!
  end

  def set_slug
    self.slug = self.title.to_slug.normalize.to_s
  end

  def destroy_flagged_assets
    assets.where(delete_asset: true).destroy
  end

  def flag_used_assets
    assets.each do |asset|
      asset.used = !!self.body.match(asset.attachment.file.filename)

      unless asset.used?
        asset.used = !!self.fragments.detect do |fragment|
          !!fragment.body.match(asset.attachment.file.filename)
        end
      end
    end
  end

end
